/*
 * Form text field filling with JavaScript:
 * Fill a form text field with a value using JavaScript
 *
 * Create a text field for displaying a date. Using a JavaScript action display
 * the current date in the form field whenever the page is opened.
 *
 * Required software: PDFlib/PDFlib+PDI/PPS 9
 * Required data: none
 */
package com.pdflib.cookbook.pdflib.form_fields;

import com.pdflib.pdflib;
import com.pdflib.PDFlibException;

public class form_textfield_fill_with_js
{
    public static void main (String argv[])
    {
        /* This is where the data files are. Adjust as necessary */
        String searchpath = "../input";
        String outfile = "form_textfield_fill_with_js.pdf";
        String title = "Form Text Field Filling with JavaScript";

        pdflib p = null;
        int exitcode = 0;

        String optlist;
        int font;
        double width=160, height=30, llx = 10, lly = 700;

        try {
            p = new pdflib();

            /* This means we must check return values of load_font() etc. */
            p.set_option("errorpolicy=return");

            p.set_option("searchpath={" + searchpath + "}");

            if (p.begin_document(outfile, "") == -1)
                throw new Exception("Error: " + p.get_errmsg());

            p.set_info("Creator", "PDFlib Cookbook");
            p.set_info("Title", title);

            font = p.load_font("NotoSerif-Regular", "winansi", 
                               "simplefont embedding nosubsetting");
            if (font == -1)
                throw new Exception("Error: " + p.get_errmsg());

            /* --------------------------------------------------------------------
             * Define a JavaScript script for entering the current date in the form
             * field "date" and supply it as the open action when starting the page
             * --------------------------------------------------------------------
             *
             * First create an action of type JavaScript. With the "script" option
             * in the option list of the action define a JavaScript snippet which
             * displays the current date in the "date" text field in the format
             * "mm dd yyyy", e.g. "Apr 15 2020".
             */
            optlist = "script={var d = util.printd('mmm dd yyyy', new Date());\n" +
                "var date = this.getField('date');\ndate.value = d;}";

            int show_date = p.create_action("JavaScript", optlist);

            /* In the second step create the page. In the option list supply
             * the "action" option which attaches the "show_date" action created
             * above to the page open event.
             */
            p.begin_page_ext(0, 0, "width=a4.width height=a4.height action={open " +
                show_date + "}");

            /* ---------------------------------
             * Create the form text field "date"
             * ---------------------------------
             *
             * Create a form field of type "textfield" called "date" with a
             * background color of light blue and a black border.
             * Allow a maximum of 11 characters to be entered (maxchar=11).
             * The characters are not moved out of the field if the field size is
             * reached (scrollable=false).
             * Display equidistant subfields for each character (comb=true).
             */
            optlist = "backgroundcolor={lightblue} bordercolor={gray 0} " +
                "maxchar=11 scrollable=false font=" + font + " fontsize=18";

            p.create_field(llx, lly, llx + width, lly + height, "date", "textfield",
                optlist);
            lly-=40;

            p.fit_textline("When opening the page the current date is " +
                "automatically displayed in the field.", llx, lly, 
                "fontname=NotoSerif-Regular encoding=unicode fontsize=12");

            p.end_page_ext("");

            p.end_document("");

        } catch (PDFlibException e) {
            System.err.println("PDFlib exception occurred:");
            System.err.println("[" + e.get_errnum() + "] " + e.get_apiname()
                + ": " + e.get_errmsg());
            exitcode = 1;
        } catch (Exception e) {
            System.err.println(e);
            exitcode = 1;
        } finally {
            if (p != null) {
                p.delete();
            }
            System.exit(exitcode);
        }
    }
}
