/*
 * Embedded video:
 * Create a RichMedia annotation which plays an embedded video
 *
 * This topic is provided only for compatibility with PDFlib 9 applications
 * (but note that you must remove the "mimetype" option to PDF_load_asset() to
 * use this code with PDFlib 9).
 * 
 * It is strongly recommended to use the more powerful Screen annotations
 * and Rendition actions instead of RichMedia annotations; see PDFlib
 * Cookbook topic "screen_annotations".
 *
 * Required software: PDFlib/PDFlib+PDI/PPS 10
 * (remove the "mimetype" option in PDF_load_asset() to use with PDFlib 9)
 * Required data: video file
 */

package com.pdflib.cookbook.pdflib.multimedia;

import com.pdflib.pdflib;
import com.pdflib.PDFlibException;

public class embedded_video {
    public static void main(String argv[]) {
        /* This is where the data files are. Adjust as necessary */
        String searchpath = "../input";
        String outfile = "embedded_video.pdf";
        String title = "Embedded video";

        pdflib p = null;
        int exitcode = 0;

        // Video file in MP4 format
        String videofile = "mountains.mp4";
        
        String optlist;
        int templ;
        int video;

        try {
            final double width=550, height=310;
            
            p = new pdflib();
            
            // Make sure we are using an appropriate PDFlib version
            if ((int) p.get_option("major", "") < 10) {
                System.err.println("Error: Cookbook topic '" + title + "' requires PDFlib 10");
                System.exit(1);
            }

            /* This means we must check return values of load_asset() etc. */
            p.set_option("errorpolicy=return");

            p.set_option("searchpath={" + searchpath + "}");

            if (p.begin_document(outfile, "") == -1)
                throw new Exception(
                        "Error: " + p.get_apiname() + ": " + p.get_errmsg());

            p.set_info("Creator", "PDFlib Cookbook");
            p.set_info("Title", title);
            
            // Create template for visualizing the video before activation.
            // We display only some text; you could display an image etc.
            
            templ = p.begin_template_ext(width, height, "");
                p.set_graphics_option("fillcolor=red");
                p.rect(0, 0, width, height);
                p.fill();
    
                p.fit_textline("Video", 10, 10,
                    "fontname=NotoSerif-Regular fontsize=14 " +
                    "fitmethod=meet fillcolor=black position=center boxsize={" + width + " " + height + "}");
            p.end_template_ext(0, 0);
            
            p.begin_page_ext(0, 0, "width=a4.width height=a4.height");

            // Load video file as video asset for use with RichMedia annotations.
            // Remove the "mimetype" option to use this code with PDFlib 9
            video = p.load_asset("Video", videofile, "mimetype=video/mp4");
            if (video == -1)
                throw new Exception("Error: " + p.get_errmsg());
            
            /* The option list for the RichMedia annotation specifies the
             * template as static placeholder and configures the video asset.
             */
            optlist =
                "template={normal=" + templ + "} " +
                "richmedia={" +
                    "assets={ {asset=" + video + " name=" + videofile + "} } " +
                    "configuration={type=Video instances={ {asset=" + videofile + "} }}" +
                "}";
            
            // Create the RichMedia annotation with video
            p.create_annotation(25, 300, 575, 610, "RichMedia", optlist);

            p.end_page_ext("");

            p.end_document("");
        }
        catch (PDFlibException e) {
            System.err.println("PDFlib exception occurred:");
            System.err.println("[" + e.get_errnum() + "] " + e.get_apiname()
                + ": " + e.get_errmsg());
            exitcode = 1;
        }
        catch (Exception e) {
            System.err.println(e);
            exitcode = 1;
        }
        finally {
            if (p != null) {
                p.delete();
            }
            System.exit(exitcode);
        }
    }
}
