/*
 * Create multi-column text output which may span multiple pages.
 * A Web link is inserted with Textflow options and a matchbox.
 *
 * Required software: PDFlib/PDFlib+PDI/PPS 9
 * Required data: font file
 */
package com.pdflib.cookbook.pdflib.textflow;

import com.pdflib.pdflib;
import com.pdflib.PDFlibException;

public class starter_textflow
{
    public static void main (String argv[])
    {
        pdflib p = null;

        /* This is where the data files are. Adjust as necessary. */
        String searchpath = "../input";

        String outfile = "starter_textflow.pdf";
        String title = "Starter Textflow";
        int i, tf = -1;
        String result;
        final double llx1= 50, lly1=50, urx1=250, ury1=800;
        final double llx2=300, lly2=50, urx2=500, ury2=800;
        int exitcode = 0;

        /* Repeat the dummy text to produce more contents */
        final int count = 50;

        final String optlist1 =
            "fontname=NotoSerif-Regular fontsize=10.5 encoding=unicode " +
            "leading=125% fillcolor=black alignment=justify";

        final String optlist2 =
            "fontname=NotoSerif-Regular fontsize=16 encoding=unicode " +
            "fillcolor=red charref nounderline";

        /* Dummy text for filling the columns. Soft hyphens are marked with the
         * character reference "&shy;" (character references are enabled by the
         * charref option).
         */
        final String text = 
            "Lorem ipsum dolor sit amet, consectetur adi&shy;pi&shy;sicing elit, " +
            "sed do eius&shy;mod tempor incidi&shy;dunt ut labore et dolore " +
            "magna ali&shy;qua. Ut enim ad minim ve&shy;niam, quis nostrud " +
            "exer&shy;citation ull&shy;amco la&shy;bo&shy;ris nisi ut " +
            "ali&shy;quip ex ea commodo con&shy;sequat. Duis aute irure dolor " +
            "in repre&shy;henderit in voluptate velit esse cillum dolore eu " +
            "fugiat nulla pari&shy;atur. Excep&shy;teur sint occae&shy;cat " +
            "cupi&shy;datat non proident, sunt in culpa qui officia " +
            "dese&shy;runt mollit anim id est laborum. ";

        try {
            p = new pdflib();

            p.set_option("searchpath={" + searchpath + "}");

            /* This means we must check return values of load_font() etc. */
            p.set_option("errorpolicy=return");

            if (p.begin_document(outfile, "") == -1)
            throw new Exception("Error: " + p.get_errmsg());

            p.set_info("Creator", "PDFlib Cookbook");
            p.set_info("Title", title);

            /* Create dummy text and feed it to a Textflow object
             * object with alternating options.
             */
            for (i=1; i<=count; i++)
            {
            String num = i + " ";

            tf = p.add_textflow(tf, num, optlist2);
            if (tf == -1)
                throw new Exception("Error: " + p.get_errmsg());

            /* In the first section we include a Web link with Textflow options:
                * - emit URL text and define a matchbox named "link_matchbox" around it;
                *   specify suitable Textflow options, e.g. blue text.
                * - end matchbox
                * The corresponding annotation will be created later with
                * p.create_annotation() using the matchbox name.
                */
            if (i == 1)
            {
                tf = p.add_textflow(tf, "www.pdflib.com",
                        "save fontsize=10.5 fillcolor=blue strokecolor=blue underline " +
                        "matchbox={name=link_matchbox boxheight={ascender descender}}");
                if (tf == -1)
                    throw new Exception("Error: " + p.get_errmsg());

                tf = p.add_textflow(tf,  " ", "restore matchbox=end nounderline");
                if (tf == -1)
                    throw new Exception("Error: " + p.get_errmsg());
            }

            tf = p.add_textflow(tf, text, optlist1);
            if (tf == -1)
                throw new Exception("Error: " + p.get_errmsg());
            }

            /* Loop until all of the text is placed; create new pages
             * as long as more text needs to be placed. Two columns will
             * be created on all pages.
             */
            do
            {
                /* Add "showborder" to visualize the fitbox borders */
                String optlist = "verticalalign=justify linespreadlimit=120% ";

                p.begin_page_ext(0, 0, "width=a4.width height=a4.height");

                /* Fill the first column */
                result = p.fit_textflow(tf, llx1, lly1, urx1, ury1, optlist);

                /* Fill the second column if we have more text*/
                if (!result.equals("_stop"))
                    result = p.fit_textflow(tf, llx2, lly2, urx2, ury2,optlist);

                /* Create the Web link based on the named matchbox which
                 * was created in the Textflow.
                 */
                
                /* Create a "URI" action for the link annotation */
                optlist = "url={https://www.pdflib.com}";
                int action = p.create_action("URI", optlist);
             
                /* Coordinates are ignored since we use a matchbox */
                p.create_annotation(0, 0, 0, 0, "Link",
                    "usematchbox={link_matchbox} action={activate " + action + "} linewidth=0");

                p.end_page_ext("");

                /* "_boxfull" means we must continue because there is more text;
                 * "_nextpage" is interpreted as "start new column"
                 */
            } while (result.equals("_boxfull") || result.equals("_nextpage"));

            /* Check for errors */
            if (!result.equals("_stop"))
            {
                /* "_boxempty" happens if the box is very small and doesn't
                 * hold any text at all.
                 */
                if (result.equals( "_boxempty"))
                    throw new Exception ("Error: Textflow box too small");
                else
                {
                    /* Any other return value is a user exit caused by
                     * the "return" option; this requires dedicated code to
                     * deal with.
                     */
                    throw new Exception ("User return '" + result +
                            "' found in Textflow");
                }
            }

            p.delete_textflow(tf);

            p.end_document("");

        } catch (PDFlibException e) {
            System.err.println("PDFlib exception occurred:");
            System.err.println("[" + e.get_errnum() + "] " + e.get_apiname() +
                ": " + e.get_errmsg());
            exitcode = 1;
        } catch (Exception e) {
            System.err.println(e);
            exitcode = 1;
        } finally {
            if (p != null) {
                p.delete();
            }
            System.exit(exitcode);
        }
    }
}
